@@ -9,3 +9,8 @@ from .rollback_transaction import *
 from .commit_transaction import *
 from .explain import *
 from .alter_table import *
+from .insert import *
+from .drop import *
+
+from mindsdb_sql.parser.dialects.mysql.variable import Variable
+from mindsdb_sql.parser.dialects.mindsdb.latest import Latest
