@@ -18,35 +18,42 @@ class MindsDBLexer(Lexer):
         USE, DROP, CREATE, DESCRIBE, RETRAIN,
 
         # Misc
-        SET, AUTOCOMMIT, START, TRANSACTION, COMMIT, ROLLBACK, ALTER, EXPLAIN,
+        SET, START, TRANSACTION, COMMIT, ROLLBACK, ALTER, EXPLAIN,
+        ISOLATION, LEVEL, REPEATABLE, READ, WRITE, UNCOMMITTED, COMMITTED,
+        SERIALIZABLE, ONLY, CONVERT,
 
         # Mindsdb special
 
         PREDICTOR, PREDICTORS, DATASOURCE, INTEGRATION, INTEGRATIONS,DATASOURCES,
-        STREAM, STREAMS, PUBLICATION, PUBLICATIONS, VIEW, VIEWS, DATASETS,
+        STREAM, STREAMS, PUBLICATION, PUBLICATIONS, VIEW, VIEWS, DATASETS, DATASET,
 
         LATEST, HORIZON, USING,
-        ENGINE, TRAIN, TEST, PREDICT, MODEL, PARAMETERS,
+        ENGINE, TRAIN, PREDICT, MODEL, PARAMETERS,
 
 
-        # SHOW Keywords
+        # SHOW/DDL Keywords
 
-        SHOW, SCHEMAS, DATABASES, TABLES, TABLE, FULL,
+        SHOW, SCHEMAS, SCHEMA, DATABASES, DATABASE, TABLES, TABLE, FULL,
         VARIABLES, SESSION, STATUS,
         GLOBAL, PROCEDURE, FUNCTION, INDEX, WARNINGS,
         ENGINES, CHARSET, COLLATION, PLUGINS, CHARACTER,
+        PERSIST, PERSIST_ONLY, DEFAULT,
+        IF_EXISTS,
 
 
         # SELECT Keywords
         WITH, SELECT, DISTINCT, FROM, WHERE, AS,
         LIMIT, OFFSET, ASC, DESC, NULLS_FIRST, NULLS_LAST,
         GROUP_BY, HAVING, ORDER_BY,
-        STAR,
+        STAR, FOR, UPDATE,
 
         JOIN, INNER, OUTER, CROSS, LEFT, RIGHT, ON,
 
         UNION, ALL,
 
+        # DML
+        INSERT, INTO, VALUES,
+
         # Special
         DOT, COMMA, LPAREN, RPAREN, PARAMETER,
 
@@ -54,10 +61,10 @@ class MindsDBLexer(Lexer):
         PLUS, MINUS, DIVIDE, MODULO,
         EQUALS, NEQUALS, GREATER, GEQ, LESS, LEQ,
         AND, OR, NOT, IS,
-        IN, LIKE, CONCAT, BETWEEN, WINDOW,
+        IN, LIKE, CONCAT, BETWEEN, WINDOW, OVER, PARTITION_BY,
 
         # Data types
-        CAST, ID, INTEGER, FLOAT, STRING, NULL, TRUE, FALSE,
+        CAST, ID, INTEGER, FLOAT, QUOTE_STRING, DQUOTE_STRING, NULL, TRUE, FALSE,
 
         JSON,
 
@@ -69,7 +76,6 @@ class MindsDBLexer(Lexer):
     USE = r'\bUSE\b'
     ENGINE = r'\bENGINE\b'
     TRAIN = r'\bTRAIN\b'
-    TEST = r'\bTEST\b'
     PREDICT = r'\bPREDICT\b'
     MODEL = r'\bMODEL\b'
     DROP = r'\bDROP\b'
@@ -89,24 +95,36 @@ class MindsDBLexer(Lexer):
     PUBLICATION = r'\bPUBLICATION\b'
     PUBLICATIONS = r'\bPUBLICATIONS\b'
     DATASETS = r'\bDATASETS\b'
+    DATASET = r'\bDATASET\b'
     LATEST = r'\bLATEST\b'
 
     # Misc
     SET = r'\bSET\b'
-    AUTOCOMMIT = r'\bAUTOCOMMIT\b'
     START = r'\bSTART\b'
     TRANSACTION = r'\bTRANSACTION\b'
     COMMIT = r'\bCOMMIT\b'
     ROLLBACK = r'\bROLLBACK\b'
     EXPLAIN = r'\bEXPLAIN\b'
     ALTER = r'\bALTER\b'
+    ISOLATION = r'\bISOLATION\b'
+    LEVEL = r'\bLEVEL\b'
+    REPEATABLE = r'\bREPEATABLE\b'
+    READ = r'\bREAD\b'
+    WRITE = r'\bWRITE\b'
+    UNCOMMITTED = r'\bUNCOMMITTED\b'
+    COMMITTED = r'\bCOMMITTED\b'
+    SERIALIZABLE = r'\bSERIALIZABLE\b'
+    ONLY = r'\bONLY\b'
+    CONVERT = r'\bCONVERT\b'
 
     DESCRIBE = r'\bDESCRIBE\b'
 
     # SHOW
     SHOW = r'\bSHOW\b'
     SCHEMAS = r'\bSCHEMAS\b'
+    SCHEMA = r'\bSCHEMA\b'
     DATABASES = r'\bDATABASES\b'
+    DATABASE = r'\bDATABASE\b'
     TABLES = r'\bTABLES\b'
     TABLE = r'\bTABLE\b'
     FULL = r'\bFULL\b'
@@ -124,7 +142,10 @@ class MindsDBLexer(Lexer):
     CHARACTER = r'\bCHARACTER\b'
     COLLATION = r'\bCOLLATION\b'
     PLUGINS = r'\bPLUGINS\b'
-
+    PERSIST = r'\bPERSIST\b'
+    PERSIST_ONLY = r'\bPERSIST_ONLY\b'
+    DEFAULT = r'\bDEFAULT\b'
+    IF_EXISTS = r'\bIF[\s]+EXISTS\b'
 
     # SELECT
 
@@ -145,6 +166,8 @@ class MindsDBLexer(Lexer):
     HAVING = r'\bHAVING\b'
     ORDER_BY = r'\bORDER BY\b'
     STAR = r'\*'
+    FOR = r'\bFOR\b'
+    UPDATE = r'\bUPDATE\b'
 
     JOIN = r'\bJOIN\b'
     INNER = r'\bINNER\b'
@@ -158,6 +181,11 @@ class MindsDBLexer(Lexer):
     UNION = r'\bUNION\b'
     ALL = r'\bALL\b'
 
+    # DML
+    INSERT = r'\bINSERT\b'
+    INTO = r'\bINTO\b'
+    VALUES = r'\bVALUES\b'
+
     # Special
     DOT = r'\.'
     COMMA = r','
@@ -186,6 +214,9 @@ class MindsDBLexer(Lexer):
     CONCAT = r'\|\|'
     BETWEEN = r'\bBETWEEN\b'
     WINDOW = r'\bWINDOW\b'
+    OVER = r'\bOVER\b'
+    PARTITION_BY = r'\bPARTITION BY\b'
+
 
     # Data types
     NULL = r'\bNULL\b'
@@ -206,16 +237,17 @@ class MindsDBLexer(Lexer):
         t.value = int(t.value)
         return t
 
-    @_(r'\{.*\}')
+    @_(r'\{[\s\S]*}')
     def JSON(self, t):
         t.value = json.loads(t.value)
         return t
 
-    @_(r'"[^"]*"',
-       r"'[^']*'")
-    def STRING(self, t):
-        if t.value[0] == '"':
-            t.value = t.value.strip('\"')
-        else:
-            t.value = t.value.strip('\'')
+    @_(r"'[^']*'")
+    def QUOTE_STRING(self, t):
+        t.value = t.value.strip('\'')
+        return t
+
+    @_(r'"[^"]*"')
+    def DQUOTE_STRING(self, t):
+        t.value = t.value.strip('\"')
         return t
