@@ -1,5 +1,6 @@
 from mindsdb_sql.parser.logger import ParserLogger
 from mindsdb_sql.parser.parser import SQLParser
+from mindsdb_sql.parser.ast.drop import DropDatabase
 from mindsdb_sql.parser.ast import *
 from mindsdb_sql.parser.dialects.mysql.lexer import MySQLLexer
 from mindsdb_sql.parser.dialects.mysql.variable import Variable
@@ -33,6 +34,9 @@ class MySQLParser(SQLParser):
        'describe',
        'union',
        'select',
+       'insert',
+       'drop_database',
+       'drop_view',
        )
     def query(self, p):
         return p[0]
@@ -48,6 +52,28 @@ class MySQLParser(SQLParser):
         return AlterTable(target=p.identifier,
                           arg=' '.join([p.ID0, p.ID1]))
 
+    # DROP VEW
+    @_('DROP VIEW identifier')
+    @_('DROP VIEW IF_EXISTS identifier')
+    def drop_view(self, p):
+        if_exists = hasattr(p, 'IF_EXISTS')
+        return DropView([p.identifier], if_exists=if_exists)
+
+    @_('DROP VIEW enumeration')
+    @_('DROP VIEW IF_EXISTS enumeration')
+    def drop_view(self, p):
+        if_exists = hasattr(p, 'IF_EXISTS')
+        return DropView(p.enumeration, if_exists=if_exists)
+
+    # DROP DATABASE
+    @_('DROP DATABASE identifier')
+    @_('DROP DATABASE IF_EXISTS identifier')
+    @_('DROP SCHEMA identifier')
+    @_('DROP SCHEMA IF_EXISTS identifier')
+    def drop_database(self, p):
+        if_exists = hasattr(p, 'IF_EXISTS')
+        return DropDatabase(name=p.identifier, if_exists=if_exists)
+
     # Transactions
 
     @_('START TRANSACTION')
@@ -64,13 +90,20 @@ class MySQLParser(SQLParser):
 
     # Set
 
-    @_('SET expr')
+    @_('SET expr_list')
+    @_('SET set_modifier expr_list')
     def set(self, p):
-        return Set(arg=p.expr)
+        if len(p.expr_list) == 1:
+            arg = p.expr_list[0]
+        else:
+            arg = Tuple(items=p.expr_list)
 
-    @_('SET AUTOCOMMIT')
-    def set(self, p):
-        return Set(category=p.AUTOCOMMIT)
+        if hasattr(p, 'set_modifier'):
+            category = p.set_modifier
+        else:
+            category = None
+
+        return Set(category=category, arg=arg)
 
     @_('SET ID identifier')
     def set(self, p):
@@ -78,6 +111,82 @@ class MySQLParser(SQLParser):
             raise ParsingException(f'Expected "SET names", got "SET {p.ID}"')
         return Set(category=p.ID.lower(), arg=p.identifier)
 
+    @_('GLOBAL',
+       'PERSIST',
+       'PERSIST_ONLY',
+       'SESSION',
+       )
+    def set_modifier(self, p):
+        return p[0]
+
+    # set charset
+    @_('SET charset constant')
+    @_('SET charset DEFAULT')
+    def set(self, p):
+        if hasattr(p, 'DEFAULT'):
+            arg = SpecialConstant('DEFAULT')
+        else:
+            arg = p.constant
+        return Set(category='CHARSET', arg=arg)
+
+    @_('CHARACTER SET',
+       'CHARSET',
+       )
+    def charset(self, p):
+        return p[0]
+
+    # set transaction
+    @_('SET transact_scope TRANSACTION transact_property_list')
+    def set(self, p):
+        isolation_level = None
+        access_mode = None
+        for prop in p.transact_property_list:
+            if prop['type'] == 'iso_level':
+                isolation_level = prop['value']
+            else:
+                access_mode = prop['value']
+
+        return SetTransaction(
+            isolation_level=isolation_level,
+            access_mode=access_mode,
+            scope=p.transact_scope,
+        )
+
+    @_('GLOBAL',
+       'SESSION',
+       'empty')
+    def transact_scope(self, p):
+        return p[0]
+
+    @_('transact_property_list COMMA transact_property')
+    def transact_property_list(self, p):
+        return p.transact_property_list + [p.transact_property]
+
+    @_('transact_property')
+    def transact_property_list(self, p):
+        return [p[0]]
+
+    @_('ISOLATION LEVEL transact_level',
+       'transact_access_mode')
+    def transact_property(self, p):
+        if hasattr(p, 'transact_level'):
+            return {'type': 'iso_level', 'value': p.transact_level}
+        else:
+            return {'type': 'access_mode', 'value': p.transact_access_mode}
+
+    @_('REPEATABLE READ',
+       'READ COMMITTED',
+       'READ UNCOMMITTED',
+       'SERIALIZABLE')
+    def transact_level(self, p):
+        return ' '.join([x for x in p])
+
+    @_('READ WRITE',
+       'READ ONLY')
+    def transact_access_mode(self, p):
+        return ' '.join([x for x in p])
+
+
     # Show
     @_('SHOW show_category show_condition_or_nothing')
     def show(self, p):
@@ -131,6 +240,27 @@ class MySQLParser(SQLParser):
     def show_category(self, p):
         return ' '.join([x for x in p])
 
+    # INSERT
+    @_('INSERT INTO from_table LPAREN result_columns RPAREN select')
+    @_('INSERT INTO from_table select')
+    def insert(self, p):
+        columns = getattr(p, 'result_columns', None)
+        return Insert(table=p.from_table, columns=columns, from_select=p.select)
+
+    @_('INSERT INTO from_table LPAREN result_columns RPAREN VALUES expr_list_set')
+    @_('INSERT INTO from_table VALUES expr_list_set')
+    def insert(self, p):
+        columns = getattr(p, 'result_columns', None)
+        return Insert(table=p.from_table, columns=columns, values=p.expr_list_set)
+
+    @_('expr_list_set COMMA expr_list_set')
+    def expr_list_set(self, p):
+        return p.expr_list_set0 + p.expr_list_set1
+
+    @_('LPAREN expr_list RPAREN')
+    def expr_list_set(self, p):
+        return [p.expr_list]
+
     # DESCRIBE
 
     @_('DESCRIBE identifier')
@@ -188,6 +318,13 @@ class MySQLParser(SQLParser):
         return p.enumeration
 
     # SELECT
+    @_('select FOR UPDATE')
+    def select(self, p):
+        select = p.select
+        ensure_select_keyword_order(select, 'MODE')
+        select.mode = 'FOR UPDATE'
+        return select
+
     @_('select OFFSET constant')
     def select(self, p):
         select = p.select
@@ -299,38 +436,40 @@ class MySQLParser(SQLParser):
         select.from_table = Identifier(p.TABLES)
         return select
 
-    @_('select FROM from_table')
+    @_('select FROM from_table_aliased')
     def select(self, p):
         select = p.select
         ensure_select_keyword_order(select, 'FROM')
-        select.from_table = p.from_table
+        select.from_table = p.from_table_aliased
         return select
 
-    @_('from_table join_clause from_table')
-    def from_table(self, p):
-        return Join(left=p.from_table0,
-                    right=p.from_table1,
+    @_('from_table_aliased join_clause from_table_aliased')
+    def from_table_aliased(self, p):
+        return Join(left=p.from_table_aliased0,
+                    right=p.from_table_aliased1,
                     join_type=p.join_clause)
 
-    @_('from_table COMMA from_table')
-    def from_table(self, p):
-        return Join(left=p.from_table0,
-                    right=p.from_table1,
+    @_('from_table_aliased COMMA from_table_aliased')
+    def from_table_aliased(self, p):
+        return Join(left=p.from_table_aliased0,
+                    right=p.from_table_aliased1,
                     join_type=JoinType.INNER_JOIN,
                     implicit=True)
 
-    @_('from_table join_clause from_table ON expr')
-    def from_table(self, p):
-        return Join(left=p.from_table0,
-                    right=p.from_table1,
+    @_('from_table_aliased join_clause from_table_aliased ON expr')
+    def from_table_aliased(self, p):
+        return Join(left=p.from_table_aliased0,
+                    right=p.from_table_aliased1,
                     join_type=p.join_clause,
                     condition=p.expr)
 
     @_('from_table AS identifier',
-       'from_table identifier')
-    def from_table(self, p):
+       'from_table identifier',
+       'from_table')
+    def from_table_aliased(self, p):
         entity = p.from_table
-        entity.alias = p.identifier
+        if hasattr(p, 'identifier'):
+            entity.alias = p.identifier
         return entity
 
     @_('LPAREN query RPAREN')
@@ -339,6 +478,12 @@ class MySQLParser(SQLParser):
         query.parentheses = True
         return query
 
+    # keywords for table
+    @_('PLUGINS')
+    @_('ENGINES')
+    def from_table(self, p):
+        return Identifier.from_path_str(p[0])
+
     @_('identifier')
     def from_table(self, p):
         return p.identifier
@@ -396,9 +541,41 @@ class MySQLParser(SQLParser):
     def result_column(self, p):
         return p.star
 
-    @_('expr')
+    @_('expr',
+       'function',
+       'window_function')
     def result_column(self, p):
-        return p.expr
+        return p[0]
+
+    # Window function
+    @_('function OVER LPAREN window RPAREN')
+    def window_function(self, p):
+
+        return WindowFunction(
+            function=p.function,
+            order_by=p.window.get('order_by'),
+            partition=p.window.get('partition'),
+        )
+
+    @_('window PARTITION_BY expr_list')
+    def window(self, p):
+        window = p.window
+        part_by = p.expr_list
+        if not isinstance(part_by, list):
+            part_by = [part_by]
+
+        window['partition'] = part_by
+        return window
+
+    @_('window ORDER_BY ordering_terms')
+    def window(self, p):
+        window = p.window
+        window['order_by'] = p.ordering_terms
+        return window
+
+    @_('empty')
+    def window(self, p):
+        return {}
 
     # OPERATIONS
 
@@ -414,12 +591,16 @@ class MySQLParser(SQLParser):
             p.expr.parentheses = True
         return p.expr
 
+    @_('DATABASE LPAREN RPAREN')
+    def function(self, p):
+        return Function(op=p.DATABASE, args=[])
+
     @_('ID LPAREN DISTINCT expr_list RPAREN')
-    def expr(self, p):
+    def function(self, p):
         return Function(op=p.ID, distinct=True, args=p.expr_list)
 
     @_('ID LPAREN expr_list_or_nothing RPAREN')
-    def expr(self, p):
+    def function(self, p):
         args = p.expr_list_or_nothing
         if not args:
             args = []
@@ -442,6 +623,11 @@ class MySQLParser(SQLParser):
     def expr(self, p):
         return TypeCast(arg=p.expr, type_name=str(p.ID))
 
+    @_('CONVERT LPAREN expr COMMA ID RPAREN')
+    @_('CONVERT LPAREN expr USING ID RPAREN')
+    def expr(self, p):
+        return TypeCast(arg=p.expr, type_name=str(p.ID))
+
     @_('enumeration')
     def expr_list(self, p):
         return p.enumeration
@@ -534,9 +720,10 @@ class MySQLParser(SQLParser):
     def constant(self, p):
         return Constant(value=float(p.FLOAT))
 
-    @_('STRING')
+    @_('QUOTE_STRING')
+    @_('DQUOTE_STRING')
     def constant(self, p):
-        return Constant(value=str(p.STRING))
+        return Constant(value=str(p[0]))
 
     @_('identifier DOT identifier')
     def identifier(self, p):
@@ -546,7 +733,8 @@ class MySQLParser(SQLParser):
     @_('ID',
        'CHARSET',
        'TABLES',
-       'STATUS')
+       'STATUS',
+       'DQUOTE_STRING')
     def identifier(self, p):
         value = p[0]
         return Identifier.from_path_str(value)
