@@ -30,6 +30,8 @@ class SQLParser(Parser):
        'describe',
        'union',
        'select',
+       'insert',
+       'drop_view',
        )
     def query(self, p):
         return p[0]
@@ -45,6 +47,19 @@ class SQLParser(Parser):
         return AlterTable(target=p.identifier,
                           arg=' '.join([p.ID0, p.ID1]))
 
+    # DROP VEW
+    @_('DROP VIEW identifier')
+    @_('DROP VIEW IF_EXISTS identifier')
+    def drop_view(self, p):
+        if_exists = hasattr(p, 'IF_EXISTS')
+        return DropView([p.identifier], if_exists=if_exists)
+
+    @_('DROP VIEW enumeration')
+    @_('DROP VIEW IF_EXISTS enumeration')
+    def drop_view(self, p):
+        if_exists = hasattr(p, 'IF_EXISTS')
+        return DropView(p.enumeration, if_exists=if_exists)
+
     # Transactions
 
     @_('START TRANSACTION')
@@ -61,10 +76,6 @@ class SQLParser(Parser):
 
     # Set
 
-    @_('SET AUTOCOMMIT')
-    def set(self, p):
-        return Set(category=p.AUTOCOMMIT)
-
     @_('SET expr')
     def set(self, p):
         return Set(arg=p.expr)
@@ -120,6 +131,27 @@ class SQLParser(Parser):
     def show_category(self, p):
         return ' '.join([x for x in p])
 
+    # INSERT
+    @_('INSERT INTO from_table LPAREN result_columns RPAREN select')
+    @_('INSERT INTO from_table select')
+    def insert(self, p):
+        columns = getattr(p, 'result_columns', None)
+        return Insert(table=p.from_table, columns=columns, from_select=p.select)
+
+    @_('INSERT INTO from_table LPAREN result_columns RPAREN VALUES expr_list_set')
+    @_('INSERT INTO from_table VALUES expr_list_set')
+    def insert(self, p):
+        columns = getattr(p, 'result_columns', None)
+        return Insert(table=p.from_table, columns=columns, values=p.expr_list_set)
+
+    @_('expr_list_set COMMA expr_list_set')
+    def expr_list_set(self, p):
+        return p.expr_list_set0 + p.expr_list_set1
+
+    @_('LPAREN expr_list RPAREN')
+    def expr_list_set(self, p):
+        return [p.expr_list]
+
     # DESCRIBE
 
     @_('DESCRIBE identifier')
@@ -280,38 +312,40 @@ class SQLParser(Parser):
         select.where = where_expr
         return select
 
-    @_('select FROM from_table')
+    @_('select FROM from_table_aliased')
     def select(self, p):
         select = p.select
         ensure_select_keyword_order(select, 'FROM')
-        select.from_table = p.from_table
+        select.from_table = p.from_table_aliased
         return select
 
-    @_('from_table join_clause from_table')
-    def from_table(self, p):
-        return Join(left=p.from_table0,
-                    right=p.from_table1,
+    @_('from_table_aliased join_clause from_table_aliased')
+    def from_table_aliased(self, p):
+        return Join(left=p.from_table_aliased0,
+                    right=p.from_table_aliased1,
                     join_type=p.join_clause)
 
-    @_('from_table COMMA from_table')
-    def from_table(self, p):
-        return Join(left=p.from_table0,
-                    right=p.from_table1,
+    @_('from_table_aliased COMMA from_table_aliased')
+    def from_table_aliased(self, p):
+        return Join(left=p.from_table_aliased0,
+                    right=p.from_table_aliased1,
                     join_type=JoinType.INNER_JOIN,
                     implicit=True)
 
-    @_('from_table join_clause from_table ON expr')
-    def from_table(self, p):
-        return Join(left=p.from_table0,
-                    right=p.from_table1,
+    @_('from_table_aliased join_clause from_table_aliased ON expr')
+    def from_table_aliased(self, p):
+        return Join(left=p.from_table_aliased0,
+                    right=p.from_table_aliased1,
                     join_type=p.join_clause,
                     condition=p.expr)
 
     @_('from_table AS identifier',
-       'from_table identifier')
-    def from_table(self, p):
+       'from_table identifier',
+       'from_table')
+    def from_table_aliased(self, p):
         entity = p.from_table
-        entity.alias = p.identifier
+        if hasattr(p, 'identifier'):
+            entity.alias = p.identifier
         return entity
 
     @_('LPAREN query RPAREN')
@@ -320,6 +354,12 @@ class SQLParser(Parser):
         query.parentheses = True
         return query
 
+    # keywords for table
+    @_('PLUGINS')
+    @_('ENGINES')
+    def from_table(self, p):
+        return Identifier.from_path_str(p[0])
+
     @_('identifier')
     def from_table(self, p):
         return p.identifier
@@ -377,9 +417,41 @@ class SQLParser(Parser):
     def result_column(self, p):
         return p.star
 
-    @_('expr')
+    @_('expr',
+       'function',
+       'window_function')
     def result_column(self, p):
-        return p.expr
+        return p[0]
+
+    # Window function
+    @_('function OVER LPAREN window RPAREN')
+    def window_function(self, p):
+
+        return WindowFunction(
+            function=p.function,
+            order_by=p.window.get('order_by'),
+            partition=p.window.get('partition'),
+        )
+
+    @_('window PARTITION_BY expr_list')
+    def window(self, p):
+        window = p.window
+        part_by = p.expr_list
+        if not isinstance(part_by, list):
+            part_by = [part_by]
+
+        window['partition'] = part_by
+        return window
+
+    @_('window ORDER_BY ordering_terms')
+    def window(self, p):
+        window = p.window
+        window['order_by'] = p.ordering_terms
+        return window
+
+    @_('empty')
+    def window(self, p):
+        return {}
 
     # OPERATIONS
 
@@ -396,11 +468,11 @@ class SQLParser(Parser):
         return p.expr
 
     @_('ID LPAREN DISTINCT expr_list RPAREN')
-    def expr(self, p):
+    def function(self, p):
         return Function(op=p.ID, distinct=True, args=p.expr_list)
 
     @_('ID LPAREN expr_list_or_nothing RPAREN')
-    def expr(self, p):
+    def function(self, p):
         args = p.expr_list_or_nothing
         if not args:
             args = []
@@ -515,9 +587,10 @@ class SQLParser(Parser):
     def constant(self, p):
         return Constant(value=float(p.FLOAT))
 
-    @_('STRING')
+    @_('QUOTE_STRING')
+    @_('DQUOTE_STRING')
     def constant(self, p):
-        return Constant(value=str(p.STRING))
+        return Constant(value=str(p[0]))
 
     @_('identifier DOT identifier')
     def identifier(self, p):
@@ -527,7 +600,8 @@ class SQLParser(Parser):
     @_('ID',
        'CHARSET',
        'TABLES',
-       'STATUS')
+       'STATUS',
+       'DQUOTE_STRING')
     def identifier(self, p):
         value = p[0]
         return Identifier.from_path_str(value)
